package com.njyunzhi.yunshake.Controller;

import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import com.njyunzhi.yunshake.common.BaseController;
import com.njyunzhi.yunshake.common.ResponseBean;
import com.njyunzhi.yunshake.common.StringUtils;
import com.njyunzhi.yunshake.common.WxUtils;
import com.njyunzhi.yunshake.entity.TaPerson;
import com.njyunzhi.yunshake.service.ITaPersonService;
import com.njyunzhi.yunshake.vo.WxMaAuthParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

@Api(tags = "微信小程序")
@RestController
@RequestMapping("/ma")
public class WxMaController extends BaseController {

    @Autowired
    WxUtils wxUtils;

    @Autowired
    ITaPersonService iTaPersonService;

    @PostMapping("/login")
    @ApiOperation(value="登录", notes = "登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean login(@ApiParam("登录参数") @RequestParam String code) throws Exception {
        WxMaJscode2SessionResult info = wxUtils.getService().getUserService().getSessionInfo(code);

        String openid = info.getOpenid();
        String sessionKey=info.getSessionKey();

        TaPerson taPerson = iTaPersonService.getByOpenid(openid);
        if (null == taPerson) {
            taPerson = new TaPerson();
            taPerson.setOpenid(openid);
            iTaPersonService.save(taPerson);
        }

        Map<String, Object> result = new HashMap<>();
        result.put("skey",sessionKey);
        result.put("person", taPerson);
        result.put("token", createToken(taPerson.getPersonId()));

        return ResponseBean.success(result);
    }

    @PutMapping("/auth-user")
    @ApiOperation(value="授权头像", notes = "授权头像", httpMethod = "PUT", response = TaPerson.class)
    public ResponseBean updateUserInfo(@ApiParam("授权头像的参数") @RequestBody WxMaAuthParam params,
                                       HttpServletRequest request) throws Exception {
        checkAuthParams(params);

        // 解密用户信息
        WxMaUserInfo userInfo = wxUtils.getService().getUserService().getUserInfo(params.getSessionKey(), params.getEncryptedData(), params.getIv());

        TaPerson taPerson = getCurrentPerson(request);
        if (null == taPerson) {
            throw new Exception("校验当前人员失败, 请重试");
        }

        taPerson.setAvatar(userInfo.getAvatarUrl());
        taPerson.setNickName(userInfo.getNickName());
        taPerson.setSex(Integer.parseInt(userInfo.getGender()));

        iTaPersonService.updateById(taPerson);

        return ResponseBean.success(taPerson);
    }
    //
//    @ApiOperation(value="授权手机", notes = "授权手机", httpMethod = "PUT", response = TaPerson.class)
//    @PutMapping("/auth-phone")
//    public ResponseBean updateUserPhone(@ApiParam("授权手机的参数") @RequestBody WxMaAuthParam params) throws Exception {
////        checkAuthParams(params);
//
//        // 解密
//        WxMaPhoneNumberInfo phoneNoInfo = wxUtils.getService().getUserService().getPhoneNoInfo(params.getSessionKey(), params.getEncryptedData(), params.getIv());
//
//        TaPerson taPerson = getCurrentPerson();
//        if (null == taPerson) {
//            throw new Exception("校验当前人员失败, 请重试");
//        }
//
//        taPerson.setPhone(phoneNoInfo.getPhoneNumber());
//        iTaPersonService.updateById(taPerson);
//
//        // 查询学生
//        TaStudent taStudent = iTaStudentService.getByPhone(taPerson.getPhone());
//        if (null != taStudent) {
//            taStudent.setPersonId(taPerson.getPersonId());
//            iTaStudentService.updateById(taStudent);
//        }
//
//        return ResponseBean.success(taPerson);
//    }
//
    private void checkAuthParams(WxMaAuthParam params) throws Exception {
        String signature = params.getSignature();
        String rawData = params.getRawData();
        String encryptedData = params.getEncryptedData();
        String iv = params.getIv();
        String sessionKey = params.getSessionKey();

        if (StringUtils.isEmpty(signature) || StringUtils.isEmpty(rawData) || StringUtils.isEmpty(encryptedData) || StringUtils.isEmpty(iv) || StringUtils.isEmpty(sessionKey)) {
            throw new Exception("缺失授权参数");
        }

        boolean isValid = wxUtils.getService().getUserService().checkUserInfo(sessionKey, rawData, signature);
        if (!isValid) {
            throw new Exception("校验授权参数失败, 请重试");
        }

        return;
    }
}
