package com.njyunzhi.yunshake.common;

import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTUtil;
import com.njyunzhi.yunshake.entity.TaPerson;
import com.njyunzhi.yunshake.service.ITaPersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;

@Component
public class BaseController {

    @Autowired
    ITaPersonService iTaPersonService;

    /**
     * 生成 TOKEN
     * @param personId
     * @return
     */
    protected String createToken(String personId) {
        LocalDateTime now = LocalDateTime.now();
        Date iat = Date.from(now.atZone(ZoneId.systemDefault()).toInstant());
        Date exp = Date.from(now.plusSeconds(Constants.JWT_EXPIRE_TIME).atZone(ZoneId.systemDefault()).toInstant());
        return JWT.create()
                .setPayload("personId", personId)
                .setKey(Constants.JWT_SECRET)
                .setIssuedAt(iat)
                .setExpiresAt(exp)
                .sign();
    }

    /**
     * 获取当前小程序端人员
     * @return
     */
    public TaPerson getCurrentPerson(HttpServletRequest request) throws Exception {
        String jws = request.getHeader(Constants.JWT_AUTH_HEADER);

        Object obj = JWTUtil.parseToken(jws).getPayload("personId");
        if (null == obj) {
            throw new Exception("请退出重新登录");
        }

        String personId = obj.toString();
        TaPerson taPerson = iTaPersonService.getById(personId);
        if (null == taPerson || Constants.STATUS_DELETED == taPerson.getStatus()) {
            throw new Exception("当前人员信息异常, 请退出重新登录");
        }

        return taPerson;
    }

}
