package com.njyunzhi.yunshake.interceptor;

import cn.hutool.json.JSONUtil;
import cn.hutool.jwt.JWTUtil;
import com.njyunzhi.yunshake.common.Constants;
import com.njyunzhi.yunshake.common.ResponseBean;
import com.njyunzhi.yunshake.common.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.charset.StandardCharsets;

@Slf4j
@Component
public class PermissionInterceptor implements HandlerInterceptor {


    private String getToken(HttpServletRequest request) {
        return request.getHeader(Constants.JWT_AUTH_HEADER);
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {
        log.info("Request URI: {}", request.getRequestURI());

        // JWT
        String jws = getToken(request);
        if (StringUtils.isEmpty(jws)) {
            responseTokenError(response, "鉴权失败, 请先进行登录操作");
            return false;
        }

        try {
            JWTUtil.verify(jws, Constants.JWT_SECRET);
        } catch (Exception e) {
            log.error("鉴权失败: {}", e.getMessage());
            responseTokenError(response, "鉴权失败, 请重新登录");
            return false;
        }

        return true;
    }

//    @Override
//    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object obj, ModelAndView mv) throws Exception {
//        LocalDateTime now = LocalDateTime.now();
//        Date iat = Date.from(now.atZone(ZoneId.systemDefault()).toInstant());
//        Date exp = Date.from(now.plusSeconds(Constants.JWT_EXPIRE_TIME).atZone(ZoneId.systemDefault()).toInstant());
//
//        // 刷新 TOKEN
//        String jws = getToken(request);
//        JWT jwt = JWT.of(jws);
//        jwt.setIssuedAt(iat).setExpiresAt(exp);
//
//        response.addHeader(Constants.AUTH_HEADER, jwt.sign());
//    }

    private void responseTokenError(HttpServletResponse response, String error) throws IOException {
        response.addHeader("Content-type", "application/json");
        response.getOutputStream().write(
                JSONUtil.toJsonStr(
                        ResponseBean.error(error, ResponseBean.ERROR_AUTH_FAIL)
                ).getBytes(StandardCharsets.UTF_8));
    }
}
